@@ -13,12 +13,10 @@
 # limitations under the License.
 
 from x2paddle.core.graph import GraphNode, Graph
-from x2paddle.core.fluid_code import FluidCode
 from x2paddle.decoder.onnx_shape_inference import SymbolicShapeInference
 from onnx.checker import ValidationError
 from onnx.checker import check_model
-from onnx.utils import polish_model
-from onnx import helper
+from onnx import helper, shape_inference
 from onnx.helper import get_attribute_value, make_attribute
 from onnx.shape_inference import infer_shapes
 from onnx.mapping import TENSOR_TYPE_TO_NP_TYPE
@@ -31,6 +29,7 @@ import numpy as np
 from copy import deepcopy
 import logging as _logging
 import os
+import copy
 
 default_op_domain = 'ai.onnx'
 _logger = _logging.getLogger(__name__)
@@ -43,12 +42,23 @@ class ONNXGraphNode(GraphNode):
         else:
             super(ONNXGraphNode, self).__init__(layer, layer_name)
         self.layer_type = layer.op_type
-        self.fluid_code = FluidCode()
         self.attr_map = self.get_attr_map()
         self.out_shapes = list()
         self.dtype = None
         self.which_child = {}
 
+    def get_input_index(self, input_name):
+        """
+        get the index of input_name in layer.input
+        -1 means input_name is not in the input
+        """
+        index = -1
+        for i in range(len(self.layer.input)):
+            if input_name == self.layer.input[i]:
+                index = i
+                break
+        return index
+
     def get_attr_map(self):
         """
         convert ONNX node attributes to dict
@@ -65,6 +75,12 @@ class ONNXGraphNode(GraphNode):
             return None
         return self.attr_map['value']
 
+    @property
+    def name(self):
+        if hasattr(self, 'index'):
+            return "{}_p{}".format(self.layer_name, self.index)
+        return self.layer_name
+
     def get_attribute_value(self, attr):
         """
         get_attribute_value enhanced
@@ -89,6 +105,13 @@ class ONNXGraphNode(GraphNode):
             return default
         return self.attr_map[name]
 
+    def output(self, index=0):
+        if index > 0 and len(self.layer.output) <= index:
+            raise IndexError(
+                'Output numbers of Node:{} is {} <= index:{}'.format(
+                    self.layer_name, len(self.layer.output), index))
+        return self.layer.output[index]
+
 
 class ONNXGraphDataNode(GraphNode):
     def __init__(self, layer, layer_name=None, is_global_input=False):
@@ -101,7 +124,6 @@ class ONNXGraphDataNode(GraphNode):
         else:
             self.layer_type = 'create_parameter'
         self.layer_name = layer_name
-        self.fluid_code = FluidCode()
         self.weight = None
         self.embeded_as = None
         self.which_child = {}
@@ -111,7 +133,24 @@ class ONNXGraphDataNode(GraphNode):
         if isinstance(self.layer, ValueInfoProto):
             values = self.layer.type.tensor_type.shape.dim
             out_shapes = list()
-            out_shapes.append([dim.dim_value for dim in values])
+            shape = list()
+            for dim in values:
+                if dim.dim_value == 0:
+                    shape.append(-1)
+                else:
+                    shape.append(dim.dim_value)
+            out_shapes.append(shape)
+            return out_shapes
+        elif isinstance(self.layer, TensorProto):
+            values = self.layer.dims
+            out_shapes = list()
+            shape = list()
+            for dim in values:
+                if dim == 0:
+                    shape.append(-1)
+                else:
+                    shape.append(dim)
+            out_shapes.append(shape)
             return out_shapes
         else:
             values = self.layer.dims
@@ -119,6 +158,10 @@ class ONNXGraphDataNode(GraphNode):
             out_shapes.append(values)
             return out_shapes
 
+    @property
+    def name(self):
+        return self.layer_name
+
     @property
     def dtype(self):
         if isinstance(self.layer, ValueInfoProto):
@@ -141,10 +184,15 @@ class ONNXGraph(Graph):
         print("shape inferencing ...")
         self.graph = SymbolicShapeInference.infer_shapes(
             onnx_model, fixed_input_shape=self.fixed_input_shape)
+        if self.graph is None:
+            print('[WARNING] Shape inference by ONNX offical interface.')
+            onnx_model = shape_inference.infer_shapes(onnx_model)
+            self.graph = onnx_model.graph
         print("shape inferenced.")
         self.build()
         self.collect_value_infos()
         self.allocate_shapes()
+        self.graph_name = "ONNXModel"
 
     def get_inner_nodes(self):
         """
@@ -184,7 +232,7 @@ class ONNXGraph(Graph):
                             shape = raw_input(
                                 "Shape of Input(e.g. -1,3,224,224), enter 'N' to skip: "
                             )
-                        except:
+                        except NameError:
                             shape = input(
                                 "Shape of Input(e.g. -1,3,224,224), enter 'N' to skip: "
                             )
@@ -213,11 +261,7 @@ class ONNXGraph(Graph):
         """
         generate output_nodes node of ONNX model
         """
-        inner_nodes = self.get_inner_nodes()
-        output_nodes = [value.name for value in self.graph.output]
-        for opt_data in output_nodes:
-            if opt_data not in inner_nodes:
-                self.output_nodes.append(opt_data)
+        self.output_nodes = [value.name for value in self.graph.output]
 
     def is_place_holder_nodes(self, layer):
         """
@@ -261,11 +305,10 @@ class ONNXGraph(Graph):
         for layer_name, node in self.node_map.items():
             if isinstance(node, ONNXGraphNode):
                 self.build_connection(layer_name, node)
-
         #generate topo
         super(ONNXGraph, self).build()
 
-        self.input_nodes = self.place_holder_nodes
+        self.input_nodes = copy.deepcopy(self.place_holder_nodes)
 
     def build_connection(self, layer_name, node):
         """
@@ -281,7 +324,19 @@ class ONNXGraph(Graph):
                         if opt == in_node:
                             self.connect(nd.name, layer_name)
                             flag = 1
-                            node.which_child[nd.name] = idx
+                            if nd.name in node.which_child:
+                                for n_i, n_ipt in enumerate(node.inputs):
+                                    if first_i == n_i:
+                                        continue
+                                    if n_ipt == nd.name:
+                                        new_nd_name = "{}/{}".format(nd.name,
+                                                                     n_i)
+                                        if new_nd_name not in node.which_child:
+                                            node.which_child[new_nd_name] = idx
+                                            break
+                            else:
+                                first_i = node.inputs.index(nd.name)
+                                node.which_child[nd.name] = idx
                             self.node_map[nd.name].index = 0
                             break
                     if flag == 1:
@@ -297,11 +352,15 @@ class ONNXGraph(Graph):
         if len(node.which_child) == 0:
             ipt_node = super(ONNXGraph, self).get_node(node.inputs[idx], copy)
             return ipt_node
-
         else:
             ipt_node = super(ONNXGraph, self).get_node(node.inputs[idx], copy)
-            if ipt_node.layer_name in node.which_child:
-                ipt_node.index = node.which_child[ipt_node.layer_name]
+            new_ipt_name = "{}/{}".format(ipt_node.layer_name, idx)
+            if new_ipt_name in node.which_child:
+                ipt_node.index = node.which_child[new_ipt_name]
+            else:
+                if ipt_node.layer_name in node.which_child:
+                    ipt_node.index = node.which_child[ipt_node.layer_name]
+
             return ipt_node
 
     def graph_weights(self):
@@ -346,8 +405,12 @@ class ONNXGraph(Graph):
                     #if len(value_info['shape']) == 0 or value_info[
                     #        'dtype'] is None or 0 in value_info['shape']:
                     #    #TODO add node shape inference
+                    shape = value_info['shape']
+                    for idx in range(len(shape)):
+                        if shape[idx] == 0:
+                            shape[idx] = -1
+                    node.out_shapes.append(shape)
                     node.dtype = value_info['dtype']
-                    node.out_shapes.append(value_info['shape'])
                 else:
                     node.out_shapes.append([])
 
@@ -362,10 +425,8 @@ class ONNXDecoder(object):
         check_model(onnx_model)
 
         onnx_model = self.optimize_model_skip_op(onnx_model)
-        onnx_model = self.optimize_model_strip_initializer(onnx_model)
         onnx_model = self.optimize_node_name(onnx_model)
         self.graph = ONNXGraph(onnx_model)
-        #self.onnx_model = onnx_model
 
     def build_value_refs(self, nodes):
         """
@@ -503,7 +564,7 @@ class ONNXDecoder(object):
         """
         if name == '':
             raise ValueError('name should not be empty')
-        for s in ' .*?\\/-:':
+        for s in ' .*?\\/-:;':
             name = name.replace(s, '_')
         return 'x2paddle_' + name
 
@@ -522,6 +583,9 @@ class ONNXDecoder(object):
             item.name = self.make_variable_name(item.name)
         for node in graph.node:
             node.name = node.output[0]
+            if ":" in node.name and len(
+                    node.output) > 1 and node.op_type != "LSTM":
+                node.name = node.name.split(':')[0]
             node.name = self.make_variable_name(node.name)
             for i in range(len(node.input)):
                 if node.input[i] == '':
