@@ -1,3 +1,4 @@
+# -*- coding:UTF-8 -*-
 #   Copyright (c) 2020  PaddlePaddle Authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"
@@ -14,7 +15,7 @@
 
 import torch
 import numpy as np
-from x2paddle.core.util import *
+from x2paddle.core.util import string
 
 
 def prim_Constant(mapper, graph, node):
@@ -25,15 +26,56 @@ def prim_Constant(mapper, graph, node):
         参数含义:
         %2 (常量类型由赋值类型定义，该示例中为int型): 常量赋值结果输出。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     output = list(node.outputs())[0]
     value = output.toIValue()
     output_type = output.type()
     if isinstance(value, str):
         value = string(value)
-    if str(output_type) == "Tensor":
+    if "Tensor" in str(output_type):
+        tensor_value = value
         value = "{}".format(value)
-
+        if "tensor" in value:
+            if isinstance(tensor_value, list) or isinstance(tensor_value,
+                                                            tuple):
+                name_dict = dict()
+                for i, tv in enumerate(tensor_value):
+                    output_name_i = "{}_p{}".format(output_name, i)
+                    key_i = "input{}".format(i)
+                    mapper.paddle_params[output_name_i] = tv.cpu().detach(
+                    ).numpy()
+                    graph.add_layer(
+                        "self.create_parameter",
+                        inputs={},
+                        outputs=[output_name_i],
+                        scope_name=scope_name,
+                        dtype=string(
+                            str(mapper.paddle_params[output_name_i].dtype)),
+                        shape=mapper.paddle_params[output_name_i].shape,
+                        default_initializer="paddle.nn.initializer.Constant(value=0.0)"
+                    )
+                    name_dict[key_i] = output_name_i
+                graph.add_layer(
+                    "prim.list",
+                    inputs=name_dict,
+                    outputs=[output_name],
+                    scope_name=scope_name)
+                return [], [output_name]
+            else:
+                #                 mapper.pytorch_params[output_name] = tensor_value.cpu().detach().numpy()
+                mapper.paddle_params[output_name] = tensor_value.cpu().detach(
+                ).numpy()
+                graph.add_layer(
+                    "self.create_parameter",
+                    inputs={},
+                    outputs=[output_name],
+                    scope_name=scope_name,
+                    dtype=string(str(mapper.paddle_params[output_name].dtype)),
+                    shape=mapper.paddle_params[output_name].shape,
+                    default_initializer="paddle.nn.initializer.Constant(value=0.0)"
+                )
+                return [], [output_name]
     if "inf" in str(value):
         t = str(type(value)).split("'")[1]
         if str(value).startswith("-"):
@@ -45,7 +87,11 @@ def prim_Constant(mapper, graph, node):
         value = int(math.pow(2, 31) - 1)
     mapper.attrs[output_name] = value
     graph.add_layer(
-        "prim.constant", inputs={}, outputs=[output_name], value=value)
+        "prim.constant",
+        inputs={},
+        outputs=[output_name],
+        scope_name=scope_name,
+        value=value)
     return [], [output_name]
 
 
@@ -60,6 +106,7 @@ def prim_data(mapper, graph, node):
 
     【注意】Paddle中无此用法，所以此处翻译成赋值。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     layer_outputs = [output_name]
     layer_inputs = {}
@@ -68,12 +115,55 @@ def prim_data(mapper, graph, node):
     # 获取当前节点输出的list
     current_outputs = [output_name]
     # 处理输入0，即%4336
-    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs)
+    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs,
+                        scope_name)
     layer_inputs["input"] = inputs_name[0]
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
-    graph.add_layer("prim.equal", inputs=layer_inputs, outputs=layer_outputs)
+    graph.add_layer(
+        "prim.equal",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name)
+    return current_inputs, current_outputs
+
+
+def prim_DictConstruct(mapper, graph, node):
+    """ 构建dict。
+
+    TorchScript示例:
+        %32 : Dict(str, Tensor) = prim::DictConstruct(%30, %23, %31, %29)
+        参数含义:
+        %32 (dict): 组成的字典。
+        %30 (str): key。
+        %23 (-): value。
+        %31 (str): key。
+        %29 (-): value。
+    """
+    scope_name = mapper.normalize_scope_name(node)
+    output_name = mapper._get_outputs_name(node)[0]
+    layer_outputs = [output_name]
+    layer_inputs = {}
+    layer_attrs = {}
+    inputs_name, inputs_node = mapper._get_inputs_name(node)
+    # 获取当前节点输出的list
+    current_outputs = [output_name]
+    # 处理每个输入
+    for i, input_name in enumerate(inputs_name):
+        if i % 2 == 0:
+            layer_attrs["key{}".format(int(i / 2))] = mapper.attrs[input_name]
+        else:
+            layer_inputs["value{}".format(int(i / 2))] = input_name
+    # 获取当前节点输入的list
+    current_inputs = list(layer_inputs.values())
+
+    graph.add_layer(
+        "prim.dict_construct",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name,
+        **layer_attrs)
     return current_inputs, current_outputs
 
 
@@ -86,6 +176,7 @@ def prim_GetAttr(mapper, graph, node):
         %7 (Tensor): 输入Tensor。
         %27 (Tensor): 输入Tensor。
     """
+    scope_name = mapper.normalize_scope_name(node)
     current_node = node
     field_name_list = [node.s('name')]
     while True:
@@ -102,7 +193,7 @@ def prim_GetAttr(mapper, graph, node):
         if hasattr(part_script, field_name):
             param = getattr(part_script, field_name)
             if isinstance(param, torch.Tensor):
-                param = param.detach().numpy()
+                param = param.cpu().detach().numpy()
                 if len(param.shape) == 0:
                     param = np.reshape(param, 1)
                 if str(param.dtype) == "uint8":
@@ -129,14 +220,20 @@ def prim_If(mapper, graph, node):
         %107 (bool): if判断条件。
         %input.5 (Tensor): if控制流的输出，与%output.4对应。
     """
+    scope_name = mapper.normalize_scope_name(node)
     outputs_name = mapper._get_outputs_name(node)
     node_outputs = outputs_name.copy()
     current_outputs = outputs_name.copy()
     input_node = list(node.inputs())[0].node()
     script_input_unique_id = list(node.inputs())[0].unique()
     input_node_name = mapper.outputs_info[script_input_unique_id]
-    mapper._check_input(graph, input_node, input_node_name, current_outputs)
-    graph.add_layer("prim.if", {'input': input_node_name}, node_outputs)
+    mapper._check_input(graph, input_node, input_node_name, current_outputs,
+                        scope_name)
+    graph.add_layer(
+        "prim.if",
+        inputs={'input': input_node_name},
+        outputs=node_outputs,
+        scope_name=scope_name)
     current_layer = list(graph.layers.values())[-1]
     block0 = list(node.blocks())[0]
     block0_graph, graph_inputs0 = mapper.traverse(block0, current_layer)
@@ -163,6 +260,7 @@ def prim_ListConstruct(mapper, graph, node):
         %84 (int/其他): list第一个元素信息。
         %85 (int/其他): list第二个元素信息。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     layer_outputs = [output_name]
     layer_inputs = {}
@@ -171,11 +269,18 @@ def prim_ListConstruct(mapper, graph, node):
     current_outputs = [output_name]
     # 处理每个输入
     for i, input_name in enumerate(inputs_name):
+        mapper._check_input(graph, inputs_node[i], input_name, current_outputs,
+                            scope_name)
         layer_inputs["input{}".format(i)] = input_name
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
-    graph.add_layer("prim.list", inputs=layer_inputs, outputs=layer_outputs)
+    layer_id = graph.add_layer(
+        "prim.list",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name)
+    mapper.output2id[output_name] = layer_id
     return current_inputs, current_outputs
 
 
@@ -189,6 +294,7 @@ def prim_ListUnpack(mapper, graph, node):
         %x2.4 (Tensor): 输出，list的第二个元素。
         %4354 (list): 列表。
     """
+    scope_name = mapper.normalize_scope_name(node)
     outputs_name = mapper._get_outputs_name(node)
     layer_outputs = outputs_name.copy()
     layer_inputs = {}
@@ -196,13 +302,17 @@ def prim_ListUnpack(mapper, graph, node):
     # 获取当前节点输出的list
     current_outputs = layer_outputs.copy()
     # 处理输入0，即%4354
-    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs)
+    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs,
+                        scope_name)
     layer_inputs["input"] = inputs_name[0]
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
     graph.add_layer(
-        "prim.list_unpack", inputs=layer_inputs, outputs=layer_outputs)
+        "prim.list_unpack",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name)
     mapper.split_len[list(layer_inputs.values())[0]] = len(layer_outputs)
     return current_inputs, current_outputs
 
@@ -223,6 +333,7 @@ def prim_Loop(mapper, graph, node):
        %x.3 (Tensor): 循环中修改的Tensor。
        %x (Tensor): loop循环的输出，与%x.5对应。
     """
+    scope_name = mapper.normalize_scope_name(node)
     node_outputs = mapper._get_outputs_name(node)
     loop_inputs = {}
     block = list(node.blocks())[0]
@@ -242,7 +353,7 @@ def prim_Loop(mapper, graph, node):
             loop_input_node_name = mapper.outputs_info[
                 script_loop_input_unique_id]
             mapper._check_input(graph, loop_input_node, loop_input_node_name,
-                                node_outputs)
+                                node_outputs, scope_name)
             loop_inputs['input'] = loop_input_node_name
             loop_outputs.append(block_input_node_name)
             node_outputs.append(block_input_node_name)
@@ -252,14 +363,19 @@ def prim_Loop(mapper, graph, node):
             loop_input_node_name = mapper.outputs_info[
                 script_loop_input_unique_id]
             mapper._check_input(graph, loop_input_node, loop_input_node_name,
-                                node_outputs)
+                                node_outputs, scope_name)
             graph.add_layer(
                 "prim.equal",
                 inputs={'input': loop_input_node_name},
-                outputs=[block_input_node_name])
+                outputs=[block_input_node_name],
+                scope_name=scope_name)
             node_outputs.append(block_input_node_name)
 
-    graph.add_layer("prim.loop", inputs=loop_inputs, outputs=loop_outputs)
+    graph.add_layer(
+        "prim.loop",
+        inputs=loop_inputs,
+        outputs=loop_outputs,
+        scope_name=scope_name)
     current_layer = list(graph.layers.values())[-1]
     block_graph, graph_inputs = mapper.traverse(block, current_layer)
     for i, input_name in enumerate(graph_inputs):
@@ -279,6 +395,7 @@ def prim_min(mapper, graph, node):
         %86 (list): 输入。
         %87 (int): 输出。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     layer_outputs = [output_name]
     layer_inputs = {}
@@ -286,12 +403,17 @@ def prim_min(mapper, graph, node):
     # 获取当前节点输出的list
     current_outputs = [output_name]
     # 处理输入0，即%86
-    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs)
+    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs,
+                        scope_name)
     layer_inputs["input"] = inputs_name[0]
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
-    graph.add_layer("prim.min", inputs=layer_inputs, outputs=layer_outputs)
+    graph.add_layer(
+        "prim.min",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name)
     return current_inputs, current_outputs
 
 
@@ -304,6 +426,7 @@ def prim_NumToTensor(mapper, graph, node):
         %other.2 (Tensor): 输出。
         %1736 (-): 输入。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     layer_outputs = [output_name]
     layer_inputs = {}
@@ -312,25 +435,31 @@ def prim_NumToTensor(mapper, graph, node):
     # 获取当前节点输出的list
     current_outputs = [output_name]
     # 处理输入0，即%86
-    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs)
-    if inputs_node[0].kind() == "aten::size":
+    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs,
+                        scope_name)
+    inputs_inputs_name, inputs_inputs_node = mapper._get_inputs_name(
+        inputs_node[0])
+    if inputs_node[0].kind() == "aten::size" and len(inputs_inputs_name) > 1:
         layer_inputs["input"] = inputs_name[0]
         # 获取当前节点输入的list
         current_inputs = list(layer_inputs.values())
         graph.add_layer(
-            "prim_equal", inputs=layer_inputs, outputs=layer_outputs)
+            "prim_equal",
+            inputs=layer_inputs,
+            outputs=layer_outputs,
+            scope_name=scope_name)
     else:
-        layer_inputs["value"] = inputs_name[0]
+        layer_inputs["fill_value"] = inputs_name[0]
         # 获取当前节点输入的list
         current_inputs = list(layer_inputs.values())
         input_type = list(node.inputs())[0].type()
         layer_attrs["dtype"] = input_type
-        layer_attrs["persistable"] = True
         layer_attrs["shape"] = [1]
         graph.add_layer(
-            "fluid.layers.create_global_var",
+            "paddle.full",
             inputs=layer_inputs,
             outputs=layer_outputs,
+            scope_name=scope_name,
             **layer_attrs)
     return current_inputs, current_outputs
 
@@ -343,6 +472,7 @@ def prim_RaiseException(mapper, graph, node):
         参数含义:
         %76 (str): 异常信息。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     layer_outputs = [output_name]
     layer_inputs = {}
@@ -350,13 +480,17 @@ def prim_RaiseException(mapper, graph, node):
     # 获取当前节点输出的list
     current_outputs = [output_name]
     # 处理输入0，即%76
-    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs)
+    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs,
+                        scope_name)
     layer_inputs["input"] = inputs_name[0]
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
     graph.add_layer(
-        "prim.exception", inputs=layer_inputs, outputs=layer_outputs)
+        "prim.exception",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name)
     return current_inputs, current_outputs
 
 
@@ -369,6 +503,7 @@ def prim_requires_grad(mapper, graph, node):
         %356 (bool): 输出，当前Tensor是否计算梯度。
         %tensor.31 (Tensor): 输入的Tensor。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     layer_outputs = [output_name]
     layer_inputs = {}
@@ -376,13 +511,17 @@ def prim_requires_grad(mapper, graph, node):
     # 获取当前节点输出的list
     current_outputs = [output_name]
     # 处理输入0，即%86
-    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs)
+    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs,
+                        scope_name)
     layer_inputs["input"] = inputs_name[0]
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
     graph.add_layer(
-        "prim.requires_grad", inputs=layer_inputs, outputs=layer_outputs)
+        "prim.requires_grad",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name)
     return current_inputs, current_outputs
 
 
@@ -395,6 +534,7 @@ def prim_SetAttr(mapper, graph, node):
         %260 (-): 属性名前缀。
         %277 (-): 需要设置的值。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     field_name_list = []
     tmp_node = node
@@ -416,7 +556,8 @@ def prim_SetAttr(mapper, graph, node):
     graph.add_layer(
         "prim.set_attr",
         inputs={"input": inputs_name[1]},
-        outputs=["self." + ".".join(field_name_list).replace(".", "_")])
+        outputs=["self." + ".".join(field_name_list).replace(".", "_")],
+        scope_name=scope_name)
     return [], [output_name]
 
 
@@ -429,6 +570,7 @@ def prim_shape(mapper, graph, node):
         %4701 (list): 输出，shape信息。
         %result.1 (Tensor): 需要获取shape的值。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     layer_outputs = [output_name]
     layer_inputs = {}
@@ -436,13 +578,17 @@ def prim_shape(mapper, graph, node):
     # 获取当前节点输出的list
     current_outputs = [output_name]
     # 处理输入0，即%input.8
-    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs)
+    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs,
+                        scope_name)
     layer_inputs["input"] = inputs_name[0]
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
     graph.add_layer(
-        "fluid.layers.shape", inputs=layer_inputs, outputs=layer_outputs)
+        "paddle.shape",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name)
     return current_inputs, current_outputs
 
 
@@ -456,6 +602,7 @@ def prim_TupleConstruct(mapper, graph, node):
         %x.46 (Tensor/其他): tuple第一个元素信息。
         %aux (Tensor/其他): tuple第二个元素信息。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     layer_outputs = [output_name]
     layer_inputs = {}
@@ -468,7 +615,11 @@ def prim_TupleConstruct(mapper, graph, node):
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
-    graph.add_layer("prim.tuple", inputs=layer_inputs, outputs=layer_outputs)
+    graph.add_layer(
+        "prim.tuple",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name)
     return current_inputs, current_outputs
 
 
@@ -482,18 +633,27 @@ def prim_TupleUnpack(mapper, graph, node):
         %aux.3 (Tensor/其他): 输出，tuple第二个元素信息。
         %4492 (tuple): 需要获取元素的tuple。
     """
+    scope_name = mapper.normalize_scope_name(node)
     outputs_name = mapper._get_outputs_name(node)
     layer_outputs = outputs_name
     layer_inputs = {}
+    layer_attrs = {}
     inputs_name, inputs_node = mapper._get_inputs_name(node)
+    if inputs_node[0].kind() == "prim::GetAttr":
+        layer_attrs["input"] = list(mapper.pytorch_params[inputs_name[0]])
+    else:
+        layer_inputs["input"] = inputs_name[0]
     # 获取当前节点输出的list
     current_outputs = outputs_name
-    layer_inputs["input"] = inputs_name[0]
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
     graph.add_layer(
-        "prim.tuple_unpack", inputs=layer_inputs, outputs=layer_outputs)
+        "prim.tuple_unpack",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name,
+        **layer_attrs)
     return current_inputs, current_outputs
 
 
@@ -508,6 +668,7 @@ def prim_unchecked_cast(mapper, graph, node):
 
     【注意】Paddle中无此用法，所以此处翻译成赋值。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     layer_outputs = [output_name]
     layer_inputs = {}
@@ -516,12 +677,17 @@ def prim_unchecked_cast(mapper, graph, node):
     # 获取当前节点输出的list
     current_outputs = [output_name]
     # 处理输入0，即%size.63
-    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs)
+    mapper._check_input(graph, inputs_node[0], inputs_name[0], current_outputs,
+                        scope_name)
     layer_inputs["input"] = inputs_name[0]
     # 获取当前节点输入的list
     current_inputs = list(layer_inputs.values())
 
-    graph.add_layer("prim.equal", inputs=layer_inputs, outputs=layer_outputs)
+    graph.add_layer(
+        "prim.equal",
+        inputs=layer_inputs,
+        outputs=layer_outputs,
+        scope_name=scope_name)
     return current_inputs, current_outputs
 
 
@@ -533,9 +699,14 @@ def prim_Uninitialized(mapper, graph, node):
         参数含义:
         %345 (bool): 输出，为赋值的bool。
     """
+    scope_name = mapper.normalize_scope_name(node)
     output_name = mapper._get_outputs_name(node)[0]
     output = list(node.outputs())[0]
     mapper.attrs[output_name] = None
     graph.add_layer(
-        "prim.constant", inputs={}, outputs=[output_name], value=None)
+        "prim.constant",
+        inputs={},
+        outputs=[output_name],
+        scope_name=scope_name,
+        value=None)
     return [], [output_name]
